@@ -94,6 +94,10 @@ class Audalign:
             return
         self.hash_style = hash_style
 
+    def get_hash_style(self) -> str:
+        """Gets the hash style. Is one of ["base", "panako", "panako_mod", "base_three"]"""
+        return self.hash_style
+
     def set_accuracy(self, accuracy: int) -> None:
         """
         Sets the accuracy level of audalign object
@@ -160,6 +164,7 @@ class Audalign:
 
     @staticmethod
     def _set_freq_threshold(freq_threshold: int) -> None:
+        """Sets the lower frequency threshold. Every frequency below this is ignored. Not in hertz. matplotlib.mlab.specgram units"""
         fingerprint.threshold = freq_threshold
 
     def get_freq_threshold(self):
@@ -178,6 +183,10 @@ class Audalign:
         """
         self.multiprocessing = true_or_false
 
+    def get_multiprocessing(self) -> bool:
+        """returns true if using multiprocessing, false if single threaded"""
+        return self.multiprocessing
+
     def set_num_processors(self, num_processors: int) -> None:
         """Set to none to use all processors by default if multiprocessing is true
 
@@ -186,6 +195,10 @@ class Audalign:
         """
         self.num_processors = num_processors
 
+    def get_num_processors(self) -> int:
+        """Returns number of processors used. Defaults to None, in which case uses all system processors"""
+        return self.num_processors
+
     def save_fingerprinted_files(self, filename: str) -> None:
         """
         Serializes fingerprinted files to json or pickle file
@@ -610,7 +623,7 @@ class Audalign:
             start_end_against (tuple(float, float), optional): Silences before and after start and end. (0, -1) Silences last second, (5.4, 0) silences first 5.4 seconds
             img_width (float): width of spectrogram image for recognition.
             volume_threshold (float): doesn't find stats for sections with max volume below threshold.
-            volume_floor (float): ignores volume levels below floow.
+            volume_floor (float): ignores volume levels below floor.
             vert_scaling (float): scales vertically to speed up calculations. Smaller numbers have smaller images.
             horiz_scaling (float): scales horizontally to speed up calculations. Smaller numbers have smaller images. Affects alignment granularity.
             calc_mse (bool): also calculates mse for each shift if true. If false, uses default mse 20000000
@@ -692,6 +705,8 @@ class Audalign:
             plot=plot,
             max_lags=max_lags,
             _file_audsegs=_file_audsegs,
+            use_multiprocessing=self.multiprocessing,
+            num_processes=self.num_processors,
             **kwargs,
         )
 
@@ -771,7 +786,7 @@ class Audalign:
             start_end (tuple(float, float), optional): Silences before and after start and end. (0, -1) Silences last second, (5.4, 0) silences first 5.4 seconds
             img_width (float): width of spectrogram image for recognition.
             volume_threshold (int): doesn't find stats for sections with max volume below threshold.
-            volume_floor (float): ignores volume levels below floow.
+            volume_floor (float): ignores volume levels below floor.
             vert_scaling (float): scales vertically to speed up calculations. Smaller numbers have smaller images.
             horiz_scaling (float): scales horizontally to speed up calculations. Smaller numbers have smaller images. Affects alignment granularity.
             max_lags (float, optional): Maximum lags in seconds.
@@ -880,6 +895,7 @@ class Audalign:
         img_width: float = 1.0,
         calc_mse: bool = False,
         cor_sample_rate: int = fingerprint.DEFAULT_FS,
+        max_lags: float = None,
         **kwargs,
     ):
         """matches and relative offsets for all files in directory_path using only target file,
@@ -898,12 +914,13 @@ class Audalign:
             locality (float, optional): In seconds for fingerprints, only matches files within given window sizes
             locality_filter_prop (int, float, optional): within each offset, filters locality tuples by proportion of highest confidence to tuple confidence
             volume_threshold (float, optional): volume threshold for visual recognition. Defaults to 216.
-            volume_floor (float): ignores volume levels below floow.
+            volume_floor (float): ignores volume levels below floor.
             vert_scaling (float): scales vertically to speed up calculations. Smaller numbers have smaller images.
             horiz_scaling (float): scales horizontally to speed up calculations. Smaller numbers have smaller images. Affects alignment granularity.
             img_width (float, optional): width of image comparison for visual recognition
             calc_mse (bool): also calculates mse for each shift if true. If false, uses default mse 20000000
             cor_sample_rate (int): optionally change the sample rate if using correlation
+            max_lags (float, optional): Maximum lags in seconds for correlation.
             kwargs: additional arguments for scipy.signal.find_peaks
 
         Returns:
@@ -930,6 +947,7 @@ class Audalign:
             img_width=img_width,
             calc_mse=calc_mse,
             cor_sample_rate=cor_sample_rate,
+            max_lags=max_lags,
             **kwargs,
         )
 
@@ -974,7 +992,7 @@ class Audalign:
             max_lags (float, optional): Maximum lags in seconds for correlation.
             strength_stat (str, optional): confidence for fingerprints, ssim for visual, mse or count also work for visual. Defaults to None.
             volume_threshold (float, optional): volume threshold for visual recognition. Defaults to 216.
-            volume_floor (float): ignores volume levels below floow.
+            volume_floor (float): ignores volume levels below floor.
             vert_scaling (float): scales vertically to speed up calculations. Smaller numbers have smaller images.
             horiz_scaling (float): scales horizontally to speed up calculations. Smaller numbers have smaller images. Affects alignment granularity.
             img_width (float, optional): width of image comparison for visual recognition
@@ -1043,7 +1061,7 @@ class Audalign:
             max_lags (float, optional): Maximum lags in seconds for correlation.
             strength_stat (str, optional): confidence for fingerprints, ssim for visual, mse or count also work for visual. Defaults to None.
             volume_threshold (float, optional): volume threshold for visual recognition. Defaults to 216.
-            volume_floor (float): ignores volume levels below floow.
+            volume_floor (float): ignores volume levels below floor.
             vert_scaling (float): scales vertically to speed up calculations. Smaller numbers have smaller images.
             horiz_scaling (float): scales horizontally to speed up calculations. Smaller numbers have smaller images. Affects alignment granularity.
             img_width (float, optional): width of image comparison for visual recognition
@@ -1114,7 +1132,7 @@ class Audalign:
             match_index (int): reorders the input results to the given match index.
             strength_stat (str): strength stat for finding proper alignments. Defaults to confidence.
             volume_threshold (float, optional): volume threshold for visual recognition. Defaults to 216.
-            volume_floor (float): ignores volume levels below floow.
+            volume_floor (float): ignores volume levels below floor.
             vert_scaling (float): scales vertically to speed up calculations. Smaller numbers have smaller images.
             horiz_scaling (float): scales horizontally to speed up calculations. Smaller numbers have smaller images. Affects alignment granularity.
             img_width (float, optional): width of image comparison for visual recognition
