@@ -3,10 +3,13 @@ from audalign.filehandler import read, find_files, get_shifted_file
 import audalign
 from pydub.exceptions import CouldntDecodeError
 import numpy as np
+import tqdm
 import time
 import os
 import scipy.signal as signal
 import matplotlib.pyplot as plt
+import multiprocessing
+from functools import partial
 
 # For locality window overlaps
 SCALING_16_BIT = 65536
@@ -54,11 +57,11 @@ def correcognize(
         )
 
     if max_lags is not None:
-        max_lags = int(calc_spec_windows(max_lags, sample_rate, technique) / 2)
+        max_lags = int(calc_spec_windows(max_lags, sample_rate, technique))
     if filter_matches is None:
         filter_matches = 0.5
     if locality is not None:
-        locality = calc_spec_windows(locality, sample_rate, technique)
+        locality = int(calc_spec_windows(locality, sample_rate, technique))
     if locality_filter_prop is None:
         locality_filter_prop = DEFAULT_LOCALITY_FILTER_PROP
     elif locality_filter_prop > 1.0:
@@ -66,41 +69,202 @@ def correcognize(
     elif locality_filter_prop < 0:
         locality_filter_prop = 0.0
 
-    print(
-        f"Comparing {os.path.basename(target_file_path)} against {os.path.basename(against_file_path)}... "
-    )
-    t = time.time()
-
-    target_array = read(
-        target_file_path, start_end=start_end_target, sample_rate=sample_rate
-    )[0]
-    against_array = read(
-        against_file_path, start_end=start_end_against, sample_rate=sample_rate
-    )[0]
-
     sos = signal.butter(
         10, fingerprint.threshold, "highpass", fs=sample_rate, output="sos"
     )
     # sos = signal.butter(10, 0.125, "hp", fs=sample_rate, output="sos")
-    target_array = signal.sosfilt(sos, target_array)
-    against_array = signal.sosfilt(sos, against_array)
-    if technique == "correlation_spectrogram":
-        target_array = fingerprint.fingerprint(
-            target_array, fs=sample_rate, retspec=True
-        ).T
-        target_array = np.clip(target_array, 0, 500)  # never louder than 500
-        against_array = fingerprint.fingerprint(
-            against_array, fs=sample_rate, retspec=True
-        ).T
-        against_array = np.clip(against_array, 0, 500)  # never louder than 500
+    target_array = get_array(
+        target_file_path,
+        start_end=start_end_target,
+        sample_rate=sample_rate,
+        _file_audsegs=None,
+        technique=technique,
+        sos=sos,
+    )
+    against_array = get_array(
+        against_file_path,
+        start_end=start_end_against,
+        sample_rate=sample_rate,
+        _file_audsegs=None,
+        technique=technique,
+        sos=sos,
+    )
+
+    t = time.time()
+    file_match = _correcognize(
+        target_array=target_array,
+        target_file_path=target_file_path,
+        against_array=against_array,
+        against_file_path=against_file_path,
+        filter_matches=filter_matches,
+        match_len_filter=match_len_filter,
+        locality=locality,
+        locality_filter_prop=locality_filter_prop,
+        sample_rate=sample_rate,
+        max_lags=max_lags,
+        plot=plot,
+        technique=technique,
+        **kwargs,
+    )
+    t = time.time() - t
+
+    result = {}
+    if file_match:
+        result["match_time"] = t
+        result["match_info"] = file_match
+        return result
+
+    return None
+
+
+def correcognize_directory(
+    target_file_path: str,
+    against_directory: str,
+    start_end: tuple = None,
+    filter_matches: float = None,
+    sample_rate: int = fingerprint.DEFAULT_FS,
+    match_len_filter: int = None,
+    locality: float = None,
+    locality_filter_prop: float = None,
+    plot: bool = False,
+    max_lags: float = None,
+    _file_audsegs: dict = None,
+    _include_filename: bool = False,
+    technique: str = "correlation",
+    use_multiprocessing: bool = True,
+    num_processes: int = None,
+    **kwargs,
+):
+    """Called from audalign correcognize_directory"""
+    assert (
+        sample_rate < 200000
+    )  # I accidentally used 441000 once... not good, big crash
+    if technique not in ["correlation", "correlation_spectrogram"]:
+        raise ValueError(
+            "technique must be either 'correlation' or 'correlation_spectrogram"
+        )
+
+    t = time.time()
+
+    if max_lags is not None:
+        max_lags = int(calc_spec_windows(max_lags, sample_rate, technique))
+    if locality is not None:
+        locality = int(calc_spec_windows(locality, sample_rate, technique))
+    if locality_filter_prop is None:
+        locality_filter_prop = DEFAULT_LOCALITY_FILTER_PROP
+    elif locality_filter_prop > 1.0:
+        locality_filter_prop = 1.0
+    elif locality_filter_prop < 0:
+        locality_filter_prop = 0.0
+    if _file_audsegs is not None and filter_matches is None:
+        filter_matches = 0.0
+    elif filter_matches is None:
+        filter_matches = 0.5
+
+    sos = signal.butter(
+        10, fingerprint.threshold, "highpass", fs=sample_rate, output="sos"
+    )
+
+    if use_multiprocessing is False:
+        target_array = get_array(
+            target_file_path,
+            start_end,
+            sample_rate=sample_rate,
+            _file_audsegs=_file_audsegs,
+            technique=technique,
+            sos=sos,
+        )
+        target_file_path = (target_file_path, target_array)
+
+    if type(against_directory) == str:
+        against_files = find_files(against_directory)
+    else:
+        against_files = zip(against_directory, ["_"] * len(against_directory))
+
+    _correcognize_dir_ = partial(
+        _correcognize_dir,
+        target_file_path=target_file_path,
+        start_end=start_end,
+        _file_audsegs=_file_audsegs,
+        sample_rate=sample_rate,
+        technique=technique,
+        sos_filter=sos,
+        filter_matches=filter_matches,
+        match_len_filter=match_len_filter,
+        locality=locality,
+        locality_filter_prop=locality_filter_prop,
+        max_lags=max_lags,
+        plot=plot,
+        **kwargs,
+    )
+
+    if use_multiprocessing == False:
+        results_list = []
+        for file_path in against_files:
+            results_list += [_correcognize_dir_(file_path)]
+    else:
+        try:
+            nprocesses = num_processes or multiprocessing.cpu_count()
+        except NotImplementedError:
+            nprocesses = 1
+        else:
+            nprocesses = 1 if nprocesses <= 0 else nprocesses
+
+        with multiprocessing.Pool(nprocesses) as pool:
+            results_list = pool.map(_correcognize_dir_, tqdm.tqdm(list(against_files)))
+            pool.close()
+            pool.join()
+
+    file_match = {}
+    for i in results_list:
+        file_match = {**file_match, **i}
+
+    t = time.time() - t
+
+    result = {}
+    if file_match:
+        result["match_time"] = t
+        result["match_info"] = file_match
+        if _include_filename:
+            if type(target_file_path) == tuple:
+                result["filename"] = target_file_path[0]
+            else:
+                result["filename"] = target_file_path
+        return result
+    return None
+
+
+def _correcognize(
+    target_array: list,
+    target_file_path: str,
+    against_array: list,
+    against_file_path: str,
+    filter_matches: float = None,
+    match_len_filter: int = None,
+    locality: float = None,
+    locality_filter_prop: float = None,
+    sample_rate: int = fingerprint.DEFAULT_FS,
+    max_lags: float = None,
+    plot: bool = False,
+    technique: str = "correlation",
+    **kwargs,
+):
+    print(
+        f"Comparing {os.path.basename(target_file_path)} against {os.path.basename(against_file_path)}... "
+    )
 
     print("Calculating correlation... ", end="")
+    indexes = (
+        find_index_arr(against_array, target_array, locality, max_lags)
+        if locality is not None
+        else []
+    )
     correlation = calc_corrs(
         against_array,
         target_array,
         locality=locality,
-        max_lags=max_lags,
         technique=technique,
+        indexes=indexes,
     )
 
     if locality is None:
@@ -113,6 +277,7 @@ def correcognize(
         locality_filter_prop=locality_filter_prop,
         locality=locality,
         technique=technique,
+        indexes_len=len(indexes),
         **kwargs,
     )
 
@@ -141,7 +306,7 @@ def correcognize(
             peaks=results_list_tuple,
         )
 
-    file_match = process_results(
+    return process_results(
         results_list=results_list_tuple,
         file_name=os.path.basename(against_file_path),
         scaling_factor=scaling_factor,
@@ -150,176 +315,93 @@ def correcognize(
         technique=technique,
     )
 
-    t = time.time() - t
 
-    result = {}
-    if file_match:
-        result["match_time"] = t
-        result["match_info"] = file_match
-        return result
-
-    return None
-
-
-def correcognize_directory(
-    target_file_path: str,
-    against_directory: str,
-    start_end: tuple = None,
-    filter_matches: float = None,
-    sample_rate: int = fingerprint.DEFAULT_FS,
-    match_len_filter: int = None,
-    locality: float = None,
-    locality_filter_prop: float = None,
-    plot: bool = False,
-    max_lags: float = None,
-    _file_audsegs: dict = None,
-    technique: str = "correlation",
+def _correcognize_dir(
+    against_file_path,
+    target_file_path,
+    start_end,
+    _file_audsegs,
+    sample_rate,
+    technique,
+    sos_filter,
+    filter_matches,
+    match_len_filter,
+    locality,
+    locality_filter_prop,
+    max_lags,
+    plot,
     **kwargs,
 ):
-    """Called from audalign correcognize_directory"""
-    assert (
-        sample_rate < 200000
-    )  # I accidentally used 441000 once... not good, big crash
-    if technique not in ["correlation", "correlation_spectrogram"]:
-        raise ValueError(
-            "technique must be either 'correlation' or 'correlation_spectrogram"
+    if type(target_file_path) == str:
+        target_array = get_array(
+            target_file_path,
+            start_end,
+            sample_rate=sample_rate,
+            _file_audsegs=_file_audsegs,
+            technique=technique,
+            sos=sos_filter,
+        )
+    else:
+        target_file_path, target_array = target_file_path
+
+    against_file_path, _ = against_file_path
+
+    if os.path.basename(against_file_path) == os.path.basename(target_file_path):
+        return {}
+    try:
+        against_array = get_array(
+            against_file_path,
+            start_end=None,
+            sample_rate=sample_rate,
+            _file_audsegs=_file_audsegs,
+            technique=technique,
+            sos=sos_filter,
+        )
+        return _correcognize(
+            target_array=target_array,
+            target_file_path=target_file_path,
+            against_array=against_array,
+            against_file_path=against_file_path,
+            filter_matches=filter_matches,
+            match_len_filter=match_len_filter,
+            locality=locality,
+            locality_filter_prop=locality_filter_prop,
+            sample_rate=sample_rate,
+            max_lags=max_lags,
+            plot=plot,
+            technique=technique,
+            **kwargs,
         )
 
-    t = time.time()
+    except CouldntDecodeError:
+        print(f'File "{against_file_path}" could not be decoded')
+        return {}
 
-    if max_lags is not None:
-        max_lags = int(calc_spec_windows(max_lags, sample_rate, technique) / 2)
-    if locality is not None:
-        locality = calc_spec_windows(locality, sample_rate, technique)
-    if locality_filter_prop is None:
-        locality_filter_prop = DEFAULT_LOCALITY_FILTER_PROP
-    elif locality_filter_prop > 1.0:
-        locality_filter_prop = 1.0
-    elif locality_filter_prop < 0:
-        locality_filter_prop = 0.0
 
+def get_array(
+    file_path,
+    start_end,
+    sample_rate,
+    _file_audsegs,
+    technique,
+    sos,
+):
     if _file_audsegs is not None:
-        target_array = _file_audsegs[target_file_path]
-        # target_array = get_shifted_file( # might want for multiprocessing in the future
-        #     target_file_path, _file_audsegs[target_file_path], sample_rate=sample_rate
-        # )
-        if filter_matches is None:
-            filter_matches = 0
+        target_array = get_shifted_file(
+            file_path,
+            _file_audsegs[file_path],
+            sample_rate=sample_rate,
+        )
     else:
-        target_array = read(
-            target_file_path, start_end=start_end, sample_rate=sample_rate
-        )[0]
-
-    if filter_matches is None:
-        filter_matches = 0.5
-
-    sos = signal.butter(
-        10, fingerprint.threshold, "highpass", fs=sample_rate, output="sos"
-    )
-    target_array = signal.sosfilt(sos, target_array)
+        target_array = read(file_path, start_end=start_end, sample_rate=sample_rate)[0]
+    if sos is not None:
+        target_array = signal.sosfilt(sos, target_array)
     if technique == "correlation_spectrogram":
         target_array = fingerprint.fingerprint(
             target_array, fs=sample_rate, retspec=True
         ).T
         target_array = np.clip(target_array, 0, 500)  # never louder than 500
-
-    if type(against_directory) == str:
-        against_files = find_files(against_directory)
-    else:
-        against_files = zip(against_directory, ["_"] * len(against_directory))
-    file_match = {}
-    for file_path, _ in against_files:
-
-        if os.path.basename(file_path) == os.path.basename(target_file_path):
-            continue
-        try:
-            print(
-                f"Comparing {os.path.basename(target_file_path)} against {os.path.basename(file_path)}... "
-            )
-            if _file_audsegs is not None:
-                against_array = _file_audsegs[file_path]
-                # against_array = get_shifted_file( # might want for multiprocessing in the future
-                #     file_path, _file_audsegs[file_path], sample_rate=sample_rate
-                # )
-            else:
-                against_array = read(file_path, sample_rate=sample_rate)[0]
-            against_array = signal.sosfilt(sos, against_array)
-            if technique == "correlation_spectrogram":
-                against_array = fingerprint.fingerprint(
-                    against_array, fs=sample_rate, retspec=True
-                ).T
-                against_array = np.clip(against_array, 0, 500)  # never louder than 500
-
-            print("Calculating correlation... ", end="")
-            correlation = calc_corrs(
-                against_array,
-                target_array,
-                locality=locality,
-                max_lags=max_lags,
-                technique=technique,
-            )
-
-            if locality is None:
-                correlation = list(correlation)[0]
-            results_list_tuple, scaling_factor = find_maxes(
-                correlation=correlation,
-                filter_matches=filter_matches,
-                match_len_filter=match_len_filter,
-                max_lags=max_lags,
-                locality_filter_prop=locality_filter_prop,
-                locality=locality,
-                technique=technique,
-                **kwargs,
-            )
-
-            if plot and locality is None:
-                plot_cor(
-                    array_a=target_array,
-                    array_b=against_array,
-                    corr_array=correlation,
-                    sample_rate=sample_rate,
-                    arr_a_title=target_file_path,
-                    arr_b_title=file_path,
-                    scaling_factor=scaling_factor,
-                    peaks=results_list_tuple,
-                    technique=technique,
-                )
-            elif plot:
-                print("\nCorrelation Plot not compatible with locality")
-                plot_cor(
-                    array_a=target_array,
-                    array_b=against_array,
-                    corr_array=None,
-                    sample_rate=sample_rate,
-                    arr_a_title=target_file_path,
-                    arr_b_title=file_path,
-                    scaling_factor=scaling_factor,
-                    peaks=results_list_tuple,
-                )
-
-            single_file_match = process_results(
-                results_list=results_list_tuple,
-                file_name=os.path.basename(file_path),
-                scaling_factor=scaling_factor,
-                sample_rate=sample_rate,
-                locality=locality,
-                technique=technique,
-            )
-            if single_file_match:
-                file_match = {**file_match, **single_file_match}
-
-        except CouldntDecodeError:
-            print(f'File "{file_path}" could not be decoded')
-
-    t = time.time() - t
-
-    result = {}
-    if file_match:
-        result["match_time"] = t
-        result["match_info"] = file_match
-        return result
-
-    return None
+    return target_array
 
 
 def calc_array_indexes(array, locality):
@@ -331,7 +413,10 @@ def calc_array_indexes(array, locality):
             index_list.append(i)
             for i in range(0, len(array) - int(locality), int(locality * OVERLAP_RATIO))
         ]
-        if len(array) - int(locality) not in index_list:
+        if (
+            len(array) - int(locality) not in index_list
+            and len(array) - int(locality) > 0
+        ):
             index_list.append(len(array) - int(locality))
     return index_list
 
@@ -374,40 +459,55 @@ def find_index_arr(against_array, target_array, locality, max_lags):
     else:
         for i in index_list_against:
             for j in index_list_target:
-                if abs(j - i) <= max_lags * 2 + locality:
+                if abs(j - i) <= max_lags + locality - 1:
                     index_pairs += [(i, j)]
     return index_pairs
 
 
 def calc_corrs(
-    against_array, target_array, locality: float, max_lags: float, technique: str
+    against_array,
+    target_array,
+    locality: float,
+    technique: str,
+    indexes: list,
 ):
     if locality is None:
         if technique == "correlation":
-            yield signal.correlate(against_array, target_array)
+            yield (
+                signal.correlate(against_array, target_array),
+                (against_array.size, target_array.size),
+            )
         elif technique == "correlation_spectrogram":
             against_array = against_array.T
             target_array = target_array.T
-            yield _calc_corrs_spec(against_array, target_array)
+            yield (
+                _calc_corrs_spec(against_array, target_array),
+                (against_array.shape[1], target_array.shape[1]),
+            )
     else:
         locality_a = len(against_array) if locality > len(against_array) else locality
         locality_b = len(target_array) if locality > len(target_array) else locality
-        indexes = find_index_arr(against_array, target_array, locality, max_lags)
         if technique == "correlation":
             for pair in indexes:
                 yield [
-                    signal.correlate(
-                        against_array[pair[0] : pair[0] + locality_a],
-                        target_array[pair[1] : pair[1] + locality_b],
+                    (
+                        signal.correlate(
+                            against_array[pair[0] : pair[0] + locality_a],
+                            target_array[pair[1] : pair[1] + locality_b],
+                        ),
+                        (locality_a, locality_b),
                     ),
                     pair,
                 ]
         elif technique == "correlation_spectrogram":
             for pair in indexes:
                 yield [
-                    _calc_corrs_spec(
-                        against_array[pair[0] : pair[0] + locality_a].T,
-                        target_array[pair[1] : pair[1] + locality_b].T,
+                    (
+                        _calc_corrs_spec(
+                            against_array[pair[0] : pair[0] + locality_a].T,
+                            target_array[pair[1] : pair[1] + locality_b].T,
+                        ),
+                        (locality_a, locality_b),
                     ),
                     pair,
                 ]
@@ -428,12 +528,14 @@ def find_maxes(
     locality_filter_prop: float,
     locality: float,
     technique: str,
+    indexes_len: int,
     **kwargs,
 ) -> list:
     print("Finding Local Maximums... ", end="")
     if locality is None:
         return _find_peaks(
-            correlation=correlation,
+            correlation=correlation[0],
+            len_tups=correlation[1],
             filter_matches=filter_matches,
             match_len_filter=match_len_filter,
             max_lags=max_lags,
@@ -442,12 +544,12 @@ def find_maxes(
         )
     else:
         total_peaks, peak_indexes = [], []
-        for i in correlation:
+        for i in tqdm.tqdm(correlation, total=indexes_len):
             total_peaks += [
                 _find_peaks(
-                    correlation=i[0],
+                    correlation=i[0][0],
+                    len_tups=i[0][1],
                     filter_matches=0,
-                    # filter_matches=filter_matches,
                     match_len_filter=match_len_filter,
                     max_lags=max_lags,
                     index_pair=i[1],
@@ -467,6 +569,7 @@ def find_maxes(
 
 def _find_peaks(
     correlation: list,
+    len_tups: tuple,
     filter_matches: float,
     match_len_filter: int,
     max_lags: float,
@@ -488,39 +591,20 @@ def _find_peaks(
         if max_corr > 0
         else correlation
     )
-    # This is quite a bit faster, but I couldn't get it to work in a timely manner.
-    # Fix this for speedup with locality and max_lags
-
-    # if max_lags is not None and index_pair is not None:
-    #     shift = index_pair[0] - index_pair[1]
-    #     # lag_indexes = [0, 0]
-    #     if len(correlation) + shift > 2 * max_lags:
-    #         correlation[int(len(correlation) / 2 + max_lags - (shift / 2)) + 1 :] = 0
-    #     if len(correlation) - shift > 2 * max_lags:
-    #         correlation[: int(len(correlation) / 2 - max_lags + (shift / 2))] = 0
-    #     # correlation = correlation[lag_indexes[0] : lag_indexes[1]]
-    if max_lags is not None and index_pair is None:
-        if len(correlation) > 2 * max_lags:
-            correlation[: int(len(correlation) / 2 - max_lags)] = 0
-            correlation[int(len(correlation) / 2 + max_lags) + 1 :] = 0
+    lag_array = signal.correlation_lags(len_tups[0], len_tups[1], mode="full")
+    if max_lags is not None:
+        shift = 0
+        if index_pair is not None:
+            shift = index_pair[0] - index_pair[1]
+        if np.max(lag_array) > max_lags - shift:
+            correlation[np.where(lag_array == (max_lags - shift))[0][0] + 1 :] = 0
+        if np.min(lag_array) < -max_lags - shift:
+            correlation[: np.where(lag_array == (-max_lags - shift))[0][0]] = 0
 
     # https://docs.scipy.org/doc/scipy/reference/generated/scipy.signal.find_peaks.html
     peaks, properties = signal.find_peaks(correlation, height=filter_matches, **kwargs)
-    peaks -= int(len(correlation) / 2)
-    peaks *= 2
+    peaks = [lag_array[x] for x in peaks]
     peaks_tuples = zip(peaks, properties["peak_heights"])
-    if max_lags is not None and index_pair is not None:
-        peaks_tuples = sorted(peaks_tuples, key=lambda x: x[0])
-        lag_indexes = [0, len(peaks_tuples)]
-        shift = index_pair[0] - index_pair[1]
-        for i, peak in enumerate(peaks_tuples):
-            if peak[0] + shift < -max_lags * 2:
-                lag_indexes[0] = i + 1
-            if peak[0] + shift > max_lags * 2:
-                lag_indexes[1] = i
-                break
-        peaks_tuples = peaks_tuples[lag_indexes[0] + 1 : lag_indexes[1]]
-
     peaks_tuples = sorted(peaks_tuples, key=lambda x: x[1], reverse=True)
 
     if match_len_filter is None:
